/**
 * PicoLCD 256x64 
 * Copyright (C) 2008 mini-box.com
 *
 * Licensed under the terms of the GNU Lesser General Public License v2.1:
 * 		http://www.opensource.org/licenses/lgpl-2.1.php
 *
 * For further information visit:
 * 		http://www.picolcd.com/
 *
 * File Authors:
 * 		Barna Csenteri
 */


#include "stdafx.h"
#include "BaseClassForCards.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CBaseClassForCards

IMPLEMENT_DYNAMIC(CBaseClassForCards, CPropertySheet)

CBaseClassForCards::CBaseClassForCards(LPCTSTR pszCaption, CWnd* pParentWnd, UINT iSelectPage)
	:CPropertySheet(pszCaption, pParentWnd, iSelectPage)
{
	m_bFlasher = false;
	m_pCardLCD		= new CCardLCD();AddPage(m_pCardLCD);
	m_pCardFlash	= new CCardFlash();AddPage(m_pCardFlash);
}

CBaseClassForCards::~CBaseClassForCards()
{
	if (m_pCardLCD)		delete m_pCardLCD;m_pCardLCD = NULL;
	if (m_pCardFlash)	delete m_pCardFlash;m_pCardFlash = NULL;
}


BEGIN_MESSAGE_MAP(CBaseClassForCards, CPropertySheet)
	//{{AFX_MSG_MAP(CBaseClassForCards)
	ON_WM_SIZE()
	ON_WM_LBUTTONDBLCLK()
	ON_WM_MENUSELECT()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CBaseClassForCards message handlers

BOOL CBaseClassForCards::OnInitDialog() 
{
	BOOL b = CPropertySheet::OnInitDialog();

	setCaption(0, "LCD");
	setCaption(1, "Flash/Eprom");

	//force creating page HWND's - otherwise MFC does this only at first selection of the page

//	this->SetActivePage(m_pCardFont);
//	this->SetActivePage(m_pCardSplash);
//	this->SetActivePage(m_pCardFlash);
//	this->SetActivePage(m_pCardLCD);

//	CheckPages(m_bFlasher);

	return b;
}

void CBaseClassForCards::CheckPages(bool flasher)
{
	m_bFlasher = flasher;

	//if (!GetSafeHwnd()) return;

	if (flasher) 
	{
		this->SetActivePage(m_pCardFlash);
		EnablePage(0, FALSE);
		EnablePage(1, TRUE);
	}
	else
	{
		this->SetActivePage(m_pCardLCD);
		EnablePage(0, TRUE);
		EnablePage(1, FALSE);
	}
}

void CBaseClassForCards::setCaption(int pos, CString str)
{
	CTabCtrl* tab = GetTabControl();
	TCHAR  lpBuffer[256];
	TCITEM tcItem;
	
	tcItem.mask = TCIF_TEXT;
   	tcItem.pszText = lpBuffer;
	strcpy(tcItem.pszText, str);
	tab->SetItem(pos, &tcItem);
}

BOOL CBaseClassForCards::DestroyWindow() 
{
	return CPropertySheet::DestroyWindow();
}

void CBaseClassForCards::OnSize(UINT nType, int cx, int cy) 
{
	CPropertySheet::OnSize(nType, cx, cy);

	CRect rect;
	GetClientRect(&rect);
	
	// Resize all pages created 
	if (GetSafeHwnd())
	{
		SetComponentSize(this,AFX_IDC_TAB_CONTROL,rect.left,rect.top,rect.Width(),rect.Height());

		rect.left += 5;
		rect.right -= 5;
		rect.top += 30;
		rect.bottom -= 5;

		/*
		if (m_pCardLCD && m_pCardLCD->GetSafeHwnd())
			for (INT i = 0; i < GetPageCount(); i++) 
			{
				CPropertyPage *page = GetPage(i);

				if (IsWindow(page->GetSafeHwnd()))	
				{
					page->InvalidateRect(rect);
					page->MoveWindow(rect);
				}
			}*/
	}

/*CRect pgrect;
					page->GetClientRect(&pgrect);

					CRect newrect;
					newrect.left = (rect.Width()-pgrect.Width())/2;
					if (newrect.left < 0) newrect.left = 0;
					newrect.top = (rect.Height()-pgrect.Height())/2;
					if (newrect.top < 0) newrect.top = 0;
					newrect.right  = newrect.left + pgrect.Width();
					newrect.bottom = newrect.top  + pgrect.Height();

					page->InvalidateRect(newrect);
					page->MoveWindow(newrect);*/

	UpdateWindow();
}

void CBaseClassForCards::OnOK() 
{
}

void CBaseClassForCards::OnCancel() 
{
}

void CBaseClassForCards::EnablePage (int nPage, BOOL bEnable)
{
   // if we want to enable the page
   if (bEnable)
   {
       // remove the index from the map
       m_DisabledPages.RemoveKey (nPage);
       // take out " - Disabled" from tab label
       /*CTabCtrl* pTab = GetTabControl();
       ASSERT (pTab);
       TC_ITEM ti;
       char szText[100];
       ti.mask = TCIF_TEXT;
       ti.pszText = szText;
       ti.cchTextMax = 100;
       VERIFY (pTab->GetItem (nPage, &ti));
       char * pFound = strstr (szText, " - Disabled");
       if (pFound)
       {
           *pFound = '\0';
           VERIFY (pTab->SetItem (nPage, &ti));
       }*/
   }
   // if we want to disable it
   else
   {
       // add the index to the map
       m_DisabledPages.SetAt (nPage, nPage);
       // add " - Disabled" to tab label
       /*CTabCtrl* pTab = GetTabControl();
       ASSERT (pTab);
       TC_ITEM ti;
       char szText[100];
       ti.mask = TCIF_TEXT;
       ti.pszText = szText;
       ti.cchTextMax = 100;
       VERIFY (pTab->GetItem (nPage, &ti));
       strcat (szText, " - Disabled");
       VERIFY (pTab->SetItem (nPage, &ti));*/
   }
}

BOOL CBaseClassForCards::OnNotify(WPARAM wParam, LPARAM lParam, LRESULT* pResult)
{
	NMHDR* pnmh = (NMHDR*)lParam;
	// tab is about to change
	if (TCN_SELCHANGING == pnmh->code)
	   // save the current page index
	   m_nLastActive = GetActiveIndex ();
	// tab has been changed
	else if (TCN_SELCHANGE == pnmh->code)
	{
	   // get the current page index
	   int nCurrentPage = GetActiveIndex ();
	   // if current page is in our map of disabled pages
	   if (m_DisabledPages.Lookup (nCurrentPage, nCurrentPage))
	   {
		   // activate the previous page
		   PostMessage (PSM_SETCURSEL, m_nLastActive);
		   AfxMessageBox("Not enabled in this mode!");
	   }
	}

	return CPropertySheet::OnNotify(wParam, lParam, pResult);
}
