/**
 * PicoLCD 256x64 
 * Copyright (C) 2008 mini-box.com
 *
 * Licensed under the terms of the GNU Lesser General Public License v2.1:
 * 		http://www.opensource.org/licenses/lgpl-2.1.php
 *
 * For further information visit:
 * 		http://www.picolcd.com/
 *
 * File Authors:
 * 		Barna Csenteri
 */

#include "stdafx.h"
#include "MiniBoxDev.h"
#include "MiniBoxDevDlg.h"
#include "CardFlash.h"
#include "ReadMinMax.h"
#include "CompareDlg.h"
#include "CompareResult.h"

// CCardFlash dialog

IMPLEMENT_DYNAMIC(CCardFlash, CPropertyPage)

CCardFlash::CCardFlash()
	: CPropertyPage(CCardFlash::IDD)
{
	m_bFlasher = true;
}

CCardFlash::~CCardFlash()
{
	DestroyWindow();
}

void CCardFlash::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_LIST_ACTION, m_listAction);
	DDX_Control(pDX, IDC_PROGRESS_STATUS, m_pBar);
	DDX_Control(pDX, IDC_STATIC_STATUS, m_sAction);
}


BEGIN_MESSAGE_MAP(CCardFlash, CPropertyPage)
	ON_LBN_DBLCLK(IDC_LIST_ACTION, &CCardFlash::OnLbnDblclkListAction)
	ON_BN_CLICKED(IDC_BUTTON_SWITCHMODE, &CCardFlash::OnBnClickedButtonSwitchmode)
END_MESSAGE_MAP()


// CCardFlash message handlers

BOOL CCardFlash::OnInitDialog()
{
	CPropertyPage::OnInitDialog();

	FillActionList(m_bFlasher);
	
	return TRUE;  // return TRUE unless you set the focus to a control
	// EXCEPTION: OCX Property Pages should return FALSE
}

void CCardFlash::FillActionList(bool flasher)
{
	m_bFlasher = flasher;

	if (m_listAction.GetSafeHwnd())
	{
		m_listAction.ResetContent();
		if (flasher)
		{
			m_listAction.AddString("Write firmware (0x2000-0x5FFF)");
			m_listAction.AddString("Write internal EEPROM");
			m_listAction.AddString("Erase flash");
			m_listAction.AddString("Read flash");
			m_listAction.AddString("Compare flash");
		}
		else
		{
			m_listAction.AddString("Write internal EEPROM");
			m_listAction.AddString("Read internal EEPROM");
			m_listAction.AddString("Compare internal EEPROM");
		}
	}

}


void CCardFlash::OnLbnDblclkListAction()
{
	int sel = m_listAction.GetCurSel();
	if (sel != -1)
	{
		if (m_bFlasher)
		{
			switch (sel)
			{
			case 0://m_listAction.AddString("Write firmware (0x2000-0x5FFF)");
				{
					char BASED_CODE szFilter[] = "Hex files (*.hex)|*.hex|";
					CFileDialog cfd(TRUE,NULL,"*.hex", OFN_OVERWRITEPROMPT | OFN_EXPLORER,  szFilter,this);
					if (cfd.DoModal()==IDOK)
					{
						RedrawWindow();
						CString file = cfd.GetPathName();
						CWaitCursor cursor;
						((CMiniBoxDevDlg*)AfxGetMainWnd())->m_pHIDInterfaceLCD->writeFile(false, TYPE_CODE_MEMORY, file, -1);
					}
				}break;
			case 1://m_listAction.AddString("Write internal EEPROM");
				{
					char BASED_CODE szFilter[] = "Hex files (*.hex)|*.hex|";
					CFileDialog cfd(TRUE,NULL,"*.hex", OFN_OVERWRITEPROMPT | OFN_EXPLORER,  szFilter,this);
					if (cfd.DoModal()==IDOK)
					{
						RedrawWindow();
						CString file = cfd.GetPathName();
						CWaitCursor cursor;
						((CMiniBoxDevDlg*)AfxGetMainWnd())->m_pHIDInterfaceLCD->writeFile(false, TYPE_EPROM_INTERNAL, file, -1);
					}
				}break;
			case 2://m_listAction.AddString("Erase flash");
				{
					CReadMinMax read(this);
					read.m_strMemMax = "x5FFF";
					read.m_strMemMin = "x2000";
					if (read.DoModal() == IDOK)
					{
						unsigned long start = getno(read.m_strMemMin);
						unsigned long end = getno(read.m_strMemMax);
						RedrawWindow();
						CWaitCursor cursor;
						((CMiniBoxDevDlg*)AfxGetMainWnd())->m_pHIDInterfaceLCD->eraseFlash(false, start, end);
					}
				}break;
			case 3://m_listAction.AddString("Read flash");
				{
					char BASED_CODE szFilter[] = "Hex files (*.hex)|*.hex|";
					CFileDialog cfd(FALSE,NULL,"read.hex", OFN_OVERWRITEPROMPT | OFN_EXPLORER,  szFilter,this);
					if (cfd.DoModal()==IDOK)
					{
						CReadMinMax read(this);
						read.m_strMemMax = "x5FFF";
						read.m_strMemMin = "x2000";

						if (read.DoModal()==IDOK)
						{
							unsigned long start = getno(read.m_strMemMin);
							unsigned long end = getno(read.m_strMemMax);
							if (start >= end)
							{
								AfxMessageBox("Start or end address eronated! (start>=end)");
								return;
							}
							RedrawWindow();
							CString file = cfd.GetPathName();
							CWaitCursor cursor;
							((CMiniBoxDevDlg*)AfxGetMainWnd())->m_pHIDInterfaceLCD->readFile(false, TYPE_CODE_MEMORY, file, start, end);
						}
					}
				}break;
			case 4://m_listAction.AddString("Compare flash");
				{
					CCompareDlg read(this);
					read.m_strMax = "x5FFF";
					read.m_strMin = "x2000";
					if (read.DoModal() == IDOK)
					{
						unsigned long start = getno(read.m_strMin);
						unsigned long end = getno(read.m_strMax);

						if (start >= end)
						{
							AfxMessageBox("Start or end address eronated! (start>=end)");
							return;
						}

						if (read.m_bFromDev)
						{
							RedrawWindow();
							CString file = read.m_strFile1;
							CWaitCursor cursor;
							((CMiniBoxDevDlg*)AfxGetMainWnd())->m_pHIDInterfaceLCD->readFile(false, TYPE_CODE_MEMORY, file, start, end);
						}

						//comparing two files
						unsigned char* data1 = new unsigned char[end];
						unsigned char* data2 = new unsigned char[end];

						if (!((CMiniBoxDevDlg*)AfxGetMainWnd())->m_pHIDInterfaceLCD->loadFile2Array(data1, end, read.m_strFile1))
						{
							AfxMessageBox("Cannot load file 1");
						}
						else
						{
							if (!((CMiniBoxDevDlg*)AfxGetMainWnd())->m_pHIDInterfaceLCD->loadFile2Array(data2, end, read.m_strFile2))
							{
								AfxMessageBox("Cannot load file 2");
							}
							else
							{
								if (read.m_bWriteDump)
								{
									FILE* f1 = fopen("f1.txt", "wt");
									if (f1)
									{
										fprintf(f1, "%s\n", read.m_strFile1);
										for (unsigned long i=0;i<end;i++)
										{
											if ((i % 32) == 0) fprintf(f1, "\n%08X: ", i);
											fprintf(f1, "%02X", data1[i]);
										}
										fclose(f1);
									}

									FILE* f2 = fopen("f2.txt", "wt");
									if (f2)
									{
										fprintf(f2, "%s\n", read.m_strFile2);
										for (unsigned long i=0;i<end;i++)
										{
											if ((i % 32) == 0) fprintf(f1, "\n%08X: ", i);
											fprintf(f2, "%02X", data2[i]);
										}
										fclose(f2);
									}		
								}

								int cnt = 0;
								CString str = "",tmp;
								for (unsigned long i=0;i<end;i++)
								{
									if (data1[i] != data2[i])
									{
										if (cnt < 15)
										{
											tmp.Format("[%08X] %02X != %02X\r\n",i, data1[i], data2[i]);
											str += tmp;
											cnt++;
										}
										else
										{
											str += "More than 15 differences found\r\nComparing stopped!\n";
											i = end;
										}
									}
								}

								if (str == "") str = "The two files are identical!";

								CCompareResult comp(this);
								comp.m_strText = str;
								comp.DoModal();
							}
						}

						delete data1;
						delete data2;
					}
				}break;
			}
		}
		else
		{
			switch (sel)
			{
			case 0://m_listAction.AddString("Write internal eprom");
				{
					unsigned long writeeprom = -1;
					char BASED_CODE szFilter[] = "Hex files (*.hex)|*.hex|";
					CFileDialog cfd(TRUE,NULL,"*.hex", OFN_OVERWRITEPROMPT | OFN_EXPLORER,  szFilter,this);
					if (cfd.DoModal()==IDOK)
					{
						RedrawWindow();
						CString file = cfd.GetPathName();
						CWaitCursor cursor;
						((CMiniBoxDevDlg*)AfxGetMainWnd())->m_pHIDInterfaceLCD->writeFile(false, TYPE_EPROM_INTERNAL, file, writeeprom);
					}
				}break;
			case 1://m_listAction.AddString("Read internal eprom");
				{
					char BASED_CODE szFilter[] = "Hex files (*.hex)|*.hex|";
					CFileDialog cfd(FALSE,NULL,"read.hex", OFN_OVERWRITEPROMPT | OFN_EXPLORER,  szFilter,this);
					if (cfd.DoModal()==IDOK)
					{
						CReadMinMax read(this);
						read.m_strMemMax = "x00FF";
						read.m_strMemMin = "x0000";

						if (read.DoModal()==IDOK)
						{
							unsigned long start = getno(read.m_strMemMin);
							unsigned long end = getno(read.m_strMemMax);

							if (start >= end)
							{
								AfxMessageBox("Start or end address eronated! (start>=end)");
								return;
							}
							RedrawWindow();
							CString file = cfd.GetPathName();
							CWaitCursor cursor;
							((CMiniBoxDevDlg*)AfxGetMainWnd())->m_pHIDInterfaceLCD->readFile(false, TYPE_EPROM_INTERNAL, file, start, end);
						}
					}
				}break;
			case 2://m_listAction.AddString("Compare internal eprom");
				{
					CCompareDlg read(this);
					read.m_strMax = "x00FF";
					read.m_strMin = "x0000";
					if (read.DoModal() == IDOK)
					{
						unsigned long start = getno(read.m_strMin);
						unsigned long end = getno(read.m_strMax);

						if (start >= end)
						{
							AfxMessageBox("Start or end address eronated! (start>=end)");
							return;
						}

						if (read.m_bFromDev)
						{
							RedrawWindow();
							CString file = read.m_strFile1;
							CWaitCursor cursor;
							((CMiniBoxDevDlg*)AfxGetMainWnd())->m_pHIDInterfaceLCD->readFile(false, TYPE_EPROM_INTERNAL, file, start, end);
						}

						//comparing two files
						unsigned char* data1 = new unsigned char[end];
						unsigned char* data2 = new unsigned char[end];

						if (!((CMiniBoxDevDlg*)AfxGetMainWnd())->m_pHIDInterfaceLCD->loadFile2Array(data1, end, read.m_strFile1))
						{
							AfxMessageBox("Cannot load file 1");
						}
						else
						{
							if (!((CMiniBoxDevDlg*)AfxGetMainWnd())->m_pHIDInterfaceLCD->loadFile2Array(data2, end, read.m_strFile2))
							{
								AfxMessageBox("Cannot load file 2");
							}
							else
							{
								if (read.m_bWriteDump)
								{
									FILE* f1 = fopen("f1.txt", "wt");
									if (f1)
									{
										fprintf(f1, "%s\n", read.m_strFile1);
										for (unsigned long i=0;i<end;i++)
										{
											if ((i % 32) == 0) fprintf(f1, "\n%08X: ", i);
											fprintf(f1, "%02X", data1[i]);
										}
										fclose(f1);
									}

									FILE* f2 = fopen("f2.txt", "wt");
									if (f2)
									{
										fprintf(f2, "%s\n", read.m_strFile2);
										for (unsigned long i=0;i<end;i++)
										{
											if ((i % 32) == 0) fprintf(f1, "\n%08X: ", i);
											fprintf(f2, "%02X", data2[i]);
										}
										fclose(f2);
									}		
								}

								int cnt = 0;
								CString str = "",tmp;
								for (unsigned long i=0;i<end;i++)
								{
									if (data1[i] != data2[i])
									{
										if (cnt < 15)
										{
											tmp.Format("[%08X] %02X != %02X\r\n",i, data1[i], data2[i]);
											str += tmp;
											cnt++;
										}
										else
										{
											str += "More than 15 differences found\r\nComparing stopped!\n";
											i = end;
										}
									}
								}

								if (str == "") str = "The two files are identical!";

								CCompareResult comp(this);
								comp.m_strText = str;
								comp.DoModal();
							}
						}

						delete data1;
						delete data2;
					}
				}break;
			}
		}
	}
}

void CCardFlash::OnBnClickedButtonSwitchmode()
{
	((CMiniBoxDevDlg*)AfxGetMainWnd())->SwitchMode();
}
