/**
 * PicoLCD 256x64 
 * Copyright (C) 2008 mini-box.com
 *
 * Licensed under the terms of the GNU Lesser General Public License v2.1:
 * 		http://www.opensource.org/licenses/lgpl-2.1.php
 *
 * For further information visit:
 * 		http://www.picolcd.com/
 *
 * File Authors:
 * 		Barna Csenteri
 */


#include "stdafx.h"
#include "MiniBoxDev.h"
#include "CardLCD.h"
#include "MiniBoxDevDlg.h"

#define CONTRAST_START 0x99

// CCardLCD dialog

IMPLEMENT_DYNAMIC(CCardLCD, CPropertyPage)

CCardLCD::CCardLCD()
	: CPropertyPage(CCardLCD::IDD)
{
	m_nTimer = 0;
	m_strLCD = "";
	m_hBmpNew = NULL; 
	m_strLastBitmapFile = "";
}

CCardLCD::~CCardLCD()
{
	DestroyWindow();
}

void CCardLCD::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_SLIDER_CONTRAST, m_sliderContrast);
	DDX_Control(pDX, IDC_SLIDER_BRIGHTNESS, m_sliderBacklight);
	DDX_Control(pDX, IDC_STATIC_KEY1, m_staticKey[0]);
	DDX_Control(pDX, IDC_STATIC_KEY2, m_staticKey[1]);
	DDX_Control(pDX, IDC_STATIC_KEY3, m_staticKey[2]);
	DDX_Control(pDX, IDC_STATIC_KEY4, m_staticKey[3]);
	DDX_Control(pDX, IDC_STATIC_KEY5, m_staticKey[4]);
	DDX_Control(pDX, IDC_STATIC_KEY6, m_staticKey[5]);
	DDX_Control(pDX, IDC_STATIC_KEY7, m_staticKey[6]);
	DDX_Control(pDX, IDC_STATIC_KEY8, m_staticKey[7]);
}


BEGIN_MESSAGE_MAP(CCardLCD, CPropertyPage)
	ON_NOTIFY(NM_RELEASEDCAPTURE, IDC_SLIDER_CONTRAST, &CCardLCD::OnNMReleasedcaptureSliderContrast)
	ON_NOTIFY(NM_RELEASEDCAPTURE, IDC_SLIDER_BRIGHTNESS, &CCardLCD::OnNMReleasedcaptureSliderBrightness)
	ON_WM_TIMER()
	ON_WM_SHOWWINDOW()
	ON_WM_PAINT()
	ON_BN_CLICKED(IDC_BUTTON_RELOAD, &CCardLCD::OnBnClickedButtonReload)
	ON_BN_CLICKED(IDC_BUTTON_TOFLASHER, &CCardLCD::OnBnClickedButtonToflasher)
	ON_BN_CLICKED(IDC_BUTTON_SPLASH, &CCardLCD::OnBnClickedButtonSplash)
END_MESSAGE_MAP()


// CCardLCD message handlers
void CCardLCD::SendInitialValues()
{
	SendContrast();
	SendBacklight();
	UpdateKeys(0,0);
	SendBitmap("test.bmp");
}

void CCardLCD::SendContrast()
{
	TRACE("DA Contrast: %x\n",m_sliderContrast.GetPos());
	if (((CMiniBoxDevDlg*)AfxGetMainWnd())->m_pHIDInterfaceLCD)
		((CMiniBoxDevDlg*)AfxGetMainWnd())->m_pHIDInterfaceLCD->setContrast(0xFF-m_sliderContrast.GetPos()+CONTRAST_START);
}

void CCardLCD::SendBacklight()
{
//	TRACE("Backlight: %d\n",0x14-m_sliderBacklight.GetPos());
	if (((CMiniBoxDevDlg*)AfxGetMainWnd())->m_pHIDInterfaceLCD)
		((CMiniBoxDevDlg*)AfxGetMainWnd())->m_pHIDInterfaceLCD->setBacklight(1-m_sliderBacklight.GetPos());
}

bool CCardLCD::loadBitmap(CString str, TSCREEN* screen)
{
	BITMAPFILEHEADER m_bfh;
	BITMAPINFOHEADER m_bih;

	// Open .BMP file
	FILE* file = _tfopen(str, _T("rb"));
	if (file != NULL)
	{
		// Read BITMAPFILEHEADER info
		fread(&m_bfh, 1, sizeof(BITMAPFILEHEADER), file);

		// Read BITMAPINFOHEADER info
		fread(&m_bih, 1, sizeof(BITMAPINFOHEADER), file);

		if (m_bih.biBitCount!=1) 
		{
			AfxMessageBox("Only monochrome bitmaps allowed!\n");
			fclose(file);
			return false;
		}

		if ((m_bih.biWidth!=256)||(m_bih.biHeight!=64) )
		{
			AfxMessageBox("Only 256x64 bitmaps allowed!\n");
			fclose(file);
			return false;
		}

		unsigned char* buffer = new unsigned char[m_bih.biSizeImage];

		fseek(file, m_bfh.bfOffBits, 0);
		unsigned long size = fread(buffer, 1, m_bih.biSizeImage, file);

		if (size != m_bih.biSizeImage)
		{
			AfxMessageBox("Error reading bitmap!\n");
			fclose(file);
			delete buffer;
			return false;
		}

		for (int y=0;y<64;y++)
			for (int x=0;x<256;x++)
				screen->screen[x][y] = false;//all black

		//draw a rectangle around the value
		//for (int y=1;y<63;y++)
		//	for (int x=1;x<255;x++)
		//		screen.screen[x][y] = true;

		int byteno, bitno;
		for (int y=0;y<64;y++)
			for (int x=0;x<256;x++)
			{//the bitmap is inverted since black = false but on bitmap pixelvalue for black is 1

				byteno = (63-y)*256+x;//y = 63 - y since bmp y=0 is in the bottom, LCD y=0 is on top
				bitno  = 7 - (byteno % 8);//bitno=7-bitno because pixel 1 is at the most important bit of byte 1
				byteno = byteno / 8;

				screen->screen[x][y] = (((buffer[byteno] >> bitno)&1) == 1);
			}

		delete buffer;

		fclose(file);

		return true;
	}
	return false;
}

bool CCardLCD::SendBitmap(CString str)
{
	if (!((CMiniBoxDevDlg*)AfxGetMainWnd())->m_pHIDInterfaceLCD) return false;

	TSCREEN screen;
	if (loadBitmap(str, &screen))
	{
		m_strLastBitmapFile = str;

		((CMiniBoxDevDlg*)AfxGetMainWnd())->m_pHIDInterfaceLCD->sendScreen(&screen);
	
		if (m_hBmpNew) 
		{
			if (DeleteObject(m_hBmpNew))
			{
				m_hBmpNew = NULL;
			}
			else
			{
				AfxMessageBox("Error deleting old bitmap!\n");
				return false;
			}
		}

		m_hBmpNew = (HBITMAP) LoadImage(
							AfxGetInstanceHandle(),   
							str,  
							IMAGE_BITMAP,        
							0,     
							0,     
							LR_LOADFROMFILE); 

		if( m_hBmpNew == NULL )
		{
			AfxMessageBox("Load Image Failed!");
		}
		else 
		{
			GetObject( m_hBmpNew , sizeof(BITMAP), &m_bmInfo );
			SelectObject(m_dcMem, m_hBmpNew );
			Invalidate();
		}

		return true;
	}

	return false;
}

void CCardLCD::UpdateKeys(int n1, int n2)
{
	m_nKeys[0] = n1;
	m_nKeys[1] = n2;

	bool keys[8];
	for (int i=0;i<8;i++)
		keys[i] = false;

	if (m_nKeys[0] != 0)
		keys[ m_nKeys[0] - 1 ] = true;

	if (m_nKeys[1] != 0)
		keys[ m_nKeys[1] - 1 ] = true;

	for (int i=0;i<8;i++)
	{
		if (keys[i])
			 m_staticKey[i].ShowWindow(SW_SHOW);
		else m_staticKey[i].ShowWindow(SW_HIDE);
	}
}

BOOL CCardLCD::OnInitDialog()
{
	CPropertyPage::OnInitDialog();

	SetControlParent(this,IDC_EDIT_TEXT);
	SetControlParent(this,IDC_SLIDER_CONTRAST);
	SetControlParent(this,IDC_SLIDER_BRIGHTNESS);
	SetControlParent(this,IDC_PROGRESS_IR);
	SetControlParent(this,IDC_CHECK_LED1);
	SetControlParent(this,IDC_CHECK_LED2);
	SetControlParent(this,IDC_CHECK_LED3);
	SetControlParent(this,IDC_CHECK_LED4);
	SetControlParent(this,IDC_CHECK_LED5);
	SetControlParent(this,IDC_CHECK_LED6);
	SetControlParent(this,IDC_CHECK_LED7);
	SetControlParent(this,IDC_CHECK_LED8);
	SetControlParent(this,IDC_BUTTON_COMMANDS);

	//both contrast and sliders are upside down so they will be converted before sending
	m_sliderContrast.SetRange(CONTRAST_START, 0xFF);
	m_sliderContrast.SetPos(CONTRAST_START+(0xFF-DEFAULT_CONTRAST));//~75%
	m_sliderBacklight.SetRange(0, 0x1);
	//m_sliderBacklight.SetPos(0);

	CClientDC dc(this);
    m_dcMem.CreateCompatibleDC( &dc );

	m_nTimer = SetTimer(1, 500, NULL);

	return TRUE;  // return TRUE unless you set the focus to a control
	// EXCEPTION: OCX Property Pages should return FALSE
}

void CCardLCD::OnNMReleasedcaptureSliderContrast(NMHDR *pNMHDR, LRESULT *pResult)
{
	SendContrast();
	*pResult = 0;
}

void CCardLCD::OnNMReleasedcaptureSliderBrightness(NMHDR *pNMHDR, LRESULT *pResult)
{
	SendBacklight();
	*pResult = 0;
}

BOOL CCardLCD::DestroyWindow()
{
	if (m_nTimer!=0) KillTimer(m_nTimer);

	return CPropertyPage::DestroyWindow();
}

void CCardLCD::OnTimer(UINT_PTR nIDEvent)
{
	if (GetSafeHwnd() && IsWindowVisible())
	{
		//TRACE("!!!!!!! Visible!\n");
	}
	//else TRACE("!!!!!!! Invisible!\n");

	CPropertyPage::OnTimer(nIDEvent);
}

void CCardLCD::OnShowWindow(BOOL bShow, UINT nStatus)
{
	CPropertyPage::OnShowWindow(bShow, nStatus);

	if (bShow && GetSafeHwnd())
		SendInitialValues();
}



void CCardLCD::OnPaint()
{
	CPaintDC dc(this); // device context for painting
	
	if (m_hBmpNew)
	{
		dc.BitBlt(95,90,265,64, &m_dcMem, 0, 0,SRCCOPY);
	}
}

void CCardLCD::OnBnClickedButtonReload()
{
	CFileDialog dlg(true, ".bmp","*.bmp",4|2,"Bmp Files (*.bmp)",this);
	if (dlg.DoModal() == IDOK)
	{
		SendBitmap(dlg.GetPathName());
	}
}

void CCardLCD::OnBnClickedButtonToflasher()
{
	((CMiniBoxDevDlg*)AfxGetMainWnd())->SwitchMode();
}

int CCardLCD::GetSplashMaxSize()
{
	return ((CMiniBoxDevDlg*)AfxGetMainWnd())->m_pHIDInterfaceLCD->getSplashMaxSize();
}

int CCardLCD::GetSplashSize()
{
	return ((CMiniBoxDevDlg*)AfxGetMainWnd())->m_pHIDInterfaceLCD->getSplashSize();
}

void CCardLCD::OnBnClickedButtonSplash()
{
	int v0 = ((CMiniBoxDevDlg*)AfxGetMainWnd())->m_pHIDInterfaceLCD->m_nVersion[0];
	int v1 = ((CMiniBoxDevDlg*)AfxGetMainWnd())->m_pHIDInterfaceLCD->m_nVersion[1];

	int version = ((v0 & 0xFF00) >> 4)* 1000 + (v0 & 0x00FF) * 100 + ((v1 & 0xFF00) >> 4)* 10 + (v1 & 0x00FF);
	
	if (version < 4)
	{
		AfxMessageBox("Logo exchange is not permitted below firmware version 00.04! Please update your firmware!");
		return;
	}

	//read the splash - jut for testing purposes
/*	if (((CMiniBoxDevDlg*)AfxGetMainWnd())->m_pHIDInterfaceLCD->readFile(false, TYPE_CODE_SPLASH, "splash_read.hex", 0, GetSplashSize()+3))
	{
		unsigned char* data = new unsigned char[GetSplashSize()+3];
		
		if (HIDMiniBoxDev::loadFile2Array(data, GetSplashSize()+3, "splash_read.hex"))
		{
			TRACE("Splash loaded=%d\n",GetSplashSize()+3);
			for (int i=0;i<100;i++)
				TRACE("%02X ",data[i]);
		}
		//else m_sAction.SetWindowText("Parsing not succesfull! (1)");
		
		delete data;
	}
	//else m_sAction.SetWindowText("Upload failed");

	if (1) return;*/

	if (m_strLastBitmapFile == "")
	{
		AfxMessageBox("No bitmap selected!");
		return;
	}

	CWaitCursor cursor;

	int max_size_splash = GetSplashMaxSize();
	unsigned short real_len = 0;
	unsigned char* data = new unsigned char[max_size_splash];
	memset(data, 0x00, max_size_splash);

	real_len = 2;//2 bytes are the length

	const unsigned char StartOfLCDInitSeq[ 4] = {0x03,0x93,0x01,0x00};
	memcpy(data+real_len, StartOfLCDInitSeq, sizeof(StartOfLCDInitSeq));
	real_len += sizeof(StartOfLCDInitSeq);
		
	const unsigned char LCDInitSequence1[4] = {0x03,0x71,0x00,0x01};
	memcpy(data+real_len, LCDInitSequence1, sizeof(LCDInitSequence1));
	real_len += sizeof(LCDInitSequence1);

	const unsigned char LCDInitSequence2[10] = {0x09,0x94,0x00,0x02,0x00,0x64,0x3F,0x00,0x64,0xC0};
	memcpy(data+real_len, LCDInitSequence2, sizeof(LCDInitSequence2));
	real_len += sizeof(LCDInitSequence2);

	const unsigned char LCDInitSequence3[10] = {0x09,0x94,0x04,0x02,0x00,0x64,0x3F,0x00,0x64,0xC0};
	memcpy(data+real_len, LCDInitSequence3, sizeof(LCDInitSequence3));
	real_len += sizeof(LCDInitSequence3);

	const unsigned char LCDInitSequence4[10] = {0x09,0x94,0x08,0x02,0x00,0x64,0x3F,0x00,0x64,0xC0};
	memcpy(data+real_len, LCDInitSequence4, sizeof(LCDInitSequence4));
	real_len += sizeof(LCDInitSequence4);

	const unsigned char LCDInitSequence5[10] = {0x09,0x94,0x0C,0x02,0x00,0x64,0x3F,0x00,0x64,0xC0};
	memcpy(data+real_len, LCDInitSequence5, sizeof(LCDInitSequence5));
	real_len += sizeof(LCDInitSequence5);

	const unsigned char StartSplash[ 4] = {0x03,0x72,0x00,0x00};
	memcpy(data+real_len, StartSplash, sizeof(StartSplash));
	real_len += sizeof(StartSplash);

	const unsigned char ClearScreen[ 2] = {0x01,0x97};
	memcpy(data+real_len, ClearScreen, sizeof(ClearScreen));
	real_len += sizeof(ClearScreen);

	unsigned char BackLight[ 3] = {0x02,0x91,0xFF};
	if (m_sliderBacklight.GetPos() != 0) BackLight[2] = 0x00;//backlight off
	memcpy(data+real_len, BackLight, sizeof(BackLight));
	real_len += sizeof(BackLight);

	unsigned char Contrast[ 3] = {0x02,0x92,0xE0};
	Contrast[2] = 0xFF-m_sliderContrast.GetPos()+CONTRAST_START;
	memcpy(data+real_len, Contrast, sizeof(Contrast));
	real_len += sizeof(Contrast);

	//sending the bitmap 
	TSCREEN screen;
	if (loadBitmap(m_strLastBitmapFile, &screen))
	{
		int start_line = 0, end_line = 8;//the start/end line inside the virtual screen (no shift)
		HArray ha[4][16];int tx, ty;int mesg_index = 0;
		for (unsigned char cs=0;cs<4;cs++)
		{
			unsigned char chip_sel = (cs << 2);//chipselect
			for (unsigned char line=0;line<end_line;line++)
			{
				mesg_index = 0;
				ha[cs][line*2].setHIDPkt(OUT_REPORT_CMD_DATA, 8+3+32, 8, chip_sel, 0x02, 0x00, COMMAND_DELAY, 0xb8|line, 0x00, COMMAND_DELAY, 0x40);
				
				//unsigned char* buf = ha[cs][line*2].getBuf();

				ha[cs][line*2].getBuf()[9+mesg_index]  = 0; //time
				ha[cs][line*2].getBuf()[10+mesg_index] = 0;//time
				ha[cs][line*2].getBuf()[11+mesg_index] = 32;//data length

				unsigned char temp = 0;
				for (int index=0;index<32;index++)
				{
					temp = 0x00;
					for (int bit=0;bit<8;bit++)
					{
						tx = cs*64+index;
						ty = (line*8+bit + 0) % SCREEN_H;
						//ty_old = (line*8+bit + 0) % SCREEN_H;
						if (!screen.screen[tx][ty])
							temp = temp | (1 << bit);
					}
					
					ha[cs][line*2].getBuf()[12+index+mesg_index] = temp;
				}

				ha[cs][line*2+1].setHIDPkt(OUT_REPORT_DATA, 4+32, 4, chip_sel | 0x01, 0x00, 0x00, 32);
				for (int index=32;index<64;index++)
				{
					temp = 0x00;
					for (int bit=0;bit<8;bit++)
					{
						if (!screen.screen[cs*64+index][(line*8+bit + 0) % SCREEN_H])
						//	||(change_pos!=0))
							temp = temp | (1 << bit);
					}
					ha[cs][line*2+1].getBuf()[5+(index-32)] = temp;
				}
			}
		}

		for (unsigned char cs=0;cs<4;cs++)
		{
			for (unsigned char line=0;line<8;line++)
			{
				if (ha[cs][line*2].getBuf() && (ha[cs][line*2].getBuf()[0] != 0))
				{
					data[real_len] = ha[cs][line*2].length();
					real_len ++;
					memcpy(data+real_len, ha[cs][line*2].getBuf(), ha[cs][line*2].length());
					real_len += ha[cs][line*2].length();

					data[real_len] = ha[cs][line*2+1].length();
					real_len ++;
					memcpy(data+real_len, ha[cs][line*2+1].getBuf(), ha[cs][line*2+1].length());
					real_len += ha[cs][line*2+1].length();
				}
			}
		}
	}
	else 
	{
		AfxMessageBox("Splash writing failed - bitmap not found!");
		delete data;
		return;
	}

	//write the endless loop for the splash
	unsigned char EndlessTimer[ 4] = {0x03,0x71,0xFF,0xFF};
	memcpy(data+real_len, EndlessTimer, sizeof(EndlessTimer));
	real_len += sizeof(EndlessTimer);

	real_len --;
	memcpy(data, &real_len, 2);//A1 0A

	if (((CMiniBoxDevDlg*)AfxGetMainWnd())->m_pHIDInterfaceLCD->writeFile4Array(32, "splash_write.hex", data, real_len))
	{
		if (((CMiniBoxDevDlg*)AfxGetMainWnd())->m_pHIDInterfaceLCD->writeFile(false, TYPE_CODE_SPLASH, "splash_write.hex", -1))
		{
			CString str = "Logo screen uploaded successfully!";
			AfxMessageBox(str);
		}
		else 
		{
			CString str = "ERROR: Could not upload logo screen to LCD!";
			AfxMessageBox(str);
		}

		((CMiniBoxDevDlg*)AfxGetMainWnd())->m_pHIDInterfaceLCD->getMaxSplash();
	}

	delete data;
}
